# FileImport statuses
FILE_IMPORT_STATUS_PENDING = "PENDING"
FILE_IMPORT_STATUS_PROCESSING = "PROCESSING"
FILE_IMPORT_STATUS_COMPLETED = "COMPLETED"
FILE_IMPORT_STATUS_FAILED = "FAILED"
FILE_IMPORT_STATUS_CANCELLED = "CANCELLED"

# Report message statuses
REPORT_STATUS_ERROR = "ERROR"
REPORT_STATUS_WARNING = "WARNING"

# Report message types
REPORT_MESSAGE_MISSING = "MISSING"
REPORT_MESSAGE_UNKNOWN = "UNKNOWN"
REPORT_MESSAGE_WRONG_FORMAT = "WRONG_FORMAT"
REPORT_MESSAGE_WRONG_MIME_TYPE = "WRONG_MIME_TYPE"
REPORT_MESSAGE_NO_FILE = "NO_FILE"
REPORT_MESSAGE_INTERNAL_ERROR = "INTERNAL_ERROR"

# Supported MIME types for import
CSV_MIME_TYPE = "text/csv"
XLS_MIME_TYPE = "application/vnd.ms-excel"
XLSX_MIME_TYPE = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"