CORS_PLUGIN_KEY = "CORS"
CORS_PLUGIN_ALLOW_ORIGINS_KEY = "allow_origins"
CORS_PLUGIN_ALLOW_METHODS_KEY = "allow_methods"
CORS_PLUGIN_ALLOW_HEADERS_KEY = "allow_headers"
CORS_PLUGIN_ALLOW_CREDENTIALS_KEY = "allow_credentials"