"""
Error constants for user_role app.

Format: (HTTP_STATUS_CODE, "ERROR_CODE_STRING")
"""

UNAUTHORIZED_ROLE_ASSIGNMENT = (403, "UNAUTHORIZED_ROLE_ASSIGNMENT")
CANNOT_UPDATE_SUPER_USER_ROLES = (403, "CANNOT_UPDATE_SUPER_USER_ROLES")
SUPERVISOR_ONLY_ROLE = (403, "SUPERVISOR_ONLY_ROLE")