NOT_UUID_ERROR = (400, "NOT_UUID")
UNSAFE_URL_ERROR = (400, "UNSAFE_URL")
SEARCH_TOO_LONG_ERROR = (400, "SEARCH_TOO_LONG")
PERMISSION_DENIED_ERROR = (403, "PERMISSION_DENIED")
NOT_FOUND_ERROR = (404, "NOT_FOUND")
UNKNOWN_WEBSERVICE_ERROR = (404, "UNKNOWN_WEBSERVICE")
