# Plugin configuration keys
FILE_STORAGE_PLUGIN_KEY = "file_storage"
FILE_STORAGE_BACKEND_KEY = "backend"
FILE_STORAGE_BUCKET_KEY = "bucket"
FILE_STORAGE_ACCESS_KEY_KEY = "access_key"
FILE_STORAGE_SECRET_KEY_KEY = "secret_key"
FILE_STORAGE_REGION_KEY = "region"
FILE_STORAGE_ENDPOINT_URL_KEY = "endpoint_url"

# Default presigned URL expiration (5 minutes)
DEFAULT_PRESIGNED_URL_EXPIRES = 300
